@@ -1,23 +1,20 @@
+from __future__ import annotations
+
 import os
+import re
 import sys
-from typing import Dict, Iterable, List, Optional, Tuple
+from typing import Iterable
 
 if sys.version_info >= (3, 8):
-    from importlib.metadata import PackageNotFoundError
-    from importlib.metadata import version as get_version
+    from importlib.metadata import distributions as get_distributions
 else:
-    from importlib_metadata import PackageNotFoundError
-    from importlib_metadata import version as get_version
+    from importlib_metadata import distributions as get_distributions
 
 
-def lines_from_file(filename: str) -> List[str]:
-    with open(filename) as f:
-        return f.read().split("\n")
-
-
-def read_pip(filename: str) -> List[str]:
+def read_pip(filename: str) -> list[str]:
     """Return lines in pip file, concatenating included requirement files."""
-    lines = lines_from_file(filename)
+    with open(filename) as f:
+        lines = [line.strip() for line in f.readlines()]
     for line in lines:
         if line.startswith("-r "):
             orig_dirpath = os.path.dirname(os.path.realpath(filename))
@@ -27,7 +24,10 @@ def read_pip(filename: str) -> List[str]:
     return lines
 
 
-def get_package_versions(lines: Iterable[str]) -> Dict[str, str]:
+VCS_RE = re.compile(r"(|[^ ]+ @ )(bzr|git|hg|svn)\+\w+://")
+
+
+def parse_pip(lines: Iterable[str]) -> dict[str, str]:
     """Return a dictionary of package versions."""
     versions = {}
     for line in lines:
@@ -36,41 +36,50 @@ def get_package_versions(lines: Iterable[str]) -> Dict[str, str]:
         if len(line) == 0 or line.startswith(("#", "-")):
             continue
 
-        if line.startswith("https://"):
+        if line.startswith(("http://", "https://")):
+            continue
+
+        if VCS_RE.match(line):
             continue
 
         full_name, version_and_extras = line.split("==", 1)
-        # Strip extras
-        name = full_name.split("[", 1)[0].lower().replace("_", "-")
+        # Strip extras and normalize
+        name = normalize_name(full_name.split("[", 1)[0])
         version = version_and_extras.split(" ", 1)[0]
         versions[name] = version
 
     return versions
 
 
-def get_mismatches(requirements_file_path: str) -> Dict[str, Tuple[str, Optional[str]]]:
+def get_installed() -> dict[str, str]:
+    return {normalize_name(d.metadata["Name"]): d.version for d in get_distributions()}
+
+
+def normalize_name(name: str) -> str:
+    return name.lower().replace("_", "-").replace(".", "-")
+
+
+def get_mismatches(requirements_file_path: str) -> dict[str, tuple[str, str | None]]:
     """Return a dictionary of requirement mismatches."""
     pip_lines = read_pip(requirements_file_path)
+    expected = parse_pip(pip_lines)
+    installed = get_installed()
 
-    expected = get_package_versions(pip_lines)
-
-    mismatches: Dict[str, Tuple[str, Optional[str]]] = {}
-    for name, version in expected.items():
-        try:
-            actual_version = get_version(name)
-        except PackageNotFoundError:
-            mismatches[name] = (version, None)
-        else:
-            if version != actual_version:
-                mismatches[name] = (version, actual_version)
+    mismatches: dict[str, tuple[str, str | None]] = {}
+    for name, expected_version in expected.items():
+        installed_version = installed.get(name)
+        if installed_version is None:
+            mismatches[name] = (expected_version, None)
+        elif installed_version != expected_version:
+            mismatches[name] = (expected_version, installed_version)
 
     return mismatches
 
 
 def print_errors(
-    errors: List[str],
-    pre_text: Optional[str] = None,
-    post_text: Optional[str] = None,
+    errors: list[str],
+    pre_text: str | None = None,
+    post_text: str | None = None,
 ) -> None:
     """Print list of errors to stderr with optional pre_text and post_text."""
     sys.stderr.write("\033[91m")  # red text
